@@ -611,10 +611,10 @@ ephy_node_view_key_press_cb (GtkTreeView *treeview,
 {
 	gboolean handled = FALSE;
 
-	if (event->keyval == GDK_space ||
-	    event->keyval == GDK_Return ||
-	    event->keyval == GDK_KP_Enter ||
-	    event->keyval == GDK_ISO_Enter)
+	if (event->keyval == GDK_KEY_space ||
+	    event->keyval == GDK_KEY_Return ||
+	    event->keyval == GDK_KEY_KP_Enter ||
+	    event->keyval == GDK_KEY_ISO_Enter)
 	{
 		if (view->priv->toggle_column >= 0)
 		{
