@@ -34,8 +34,7 @@
 #include "ephy-gui.h"
 #include "ephy-dnd.h"
 #include "ephy-prefs.h"
-
-#include "eel-gconf-extensions.h"
+#include "ephy-settings.h"
 
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
@@ -222,7 +221,7 @@ similar_deactivate_cb (GtkMenuShell *ms,
 		       GtkWidget *button)
 {
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), FALSE);
-	g_signal_emit_by_name (button, "button-release-event");
+	gtk_button_released (GTK_BUTTON (button));
 }
 
 static void
@@ -383,8 +382,7 @@ bookmark_properties_response_cb (GtkDialog *dialog,
 	switch (response_id)
 	{
 		case GTK_RESPONSE_HELP:
-			ephy_gui_help (GTK_WINDOW (dialog),
-				       "epiphany", 
+			ephy_gui_help (GTK_WIDGET (dialog),
 				       "to-edit-bookmark-properties");
 			return;
 	 	case GTK_RESPONSE_ACCEPT:
@@ -524,12 +522,12 @@ ephy_bookmark_properties_constructor (GType type,
 				       EPHY_STATE_WINDOW_SAVE_SIZE);
 	}
 	/* Lockdown */
-	lockdown = eel_gconf_get_boolean (CONF_LOCKDOWN_DISABLE_BOOKMARK_EDITING);
+	lockdown = g_settings_get_boolean (EPHY_SETTINGS_LOCKDOWN,
+					   EPHY_PREFS_LOCKDOWN_BOOKMARK_EDITING);
 
 	update_window_title (properties);
 	content_area = gtk_dialog_get_content_area (dialog);
 
-	gtk_dialog_set_has_separator (dialog, FALSE);
 	gtk_container_set_border_width (GTK_CONTAINER (properties), 5);
 	gtk_box_set_spacing (GTK_BOX (content_area), 2);
 
