@@ -19,7 +19,6 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id$
  */
 
 #include "config.h"
@@ -33,42 +32,25 @@
 
 #include "ephy-node-db.h"
 #include "ephy-shell.h"
-#include "eel-gconf-extensions.h"
 #include "ephy-file-helpers.h"
 #include "ephy-object-helpers.h"
 #include "ephy-debug.h"
-
-#include "ephy-glib-compat.h"
-
-#include <libxml/tree.h>
-#include <libxml/xmlreader.h>
-#include <libxml/globals.h>
-#include <libxml/tree.h>
-#include <libxml/xmlwriter.h>
-#include <libxslt/xslt.h>
-#include <libxslt/transform.h>
-#include <libxslt/xsltutils.h>
-
-#include <gconf/gconf-client.h>
+#include "ephy-prefs.h"
+#include "ephy-settings.h"
 
 #include <gio/gio.h>
 #include <gmodule.h>
-#include <dirent.h>
 #include <string.h>
 
-#ifdef ENABLE_PYTHON
-#include "ephy-python-extension.h"
-#include "ephy-python-loader.h"
+#ifdef ENABLE_SEED
+#include "ephy-seed-loader.h"
 #endif
 
-#define CONF_LOADED_EXTENSIONS	"/apps/epiphany/general/active_extensions"
 #define EE_GROUP		"Epiphany Extension"
 #define DOT_INI			".ephy-extension"
 #define RELOAD_DELAY		333 /* ms */
 #define RELOAD_SYNC_DELAY	1 /* seconds */
 
-#define ENABLE_LEGACY_FORMAT
-
 #define EPHY_EXTENSIONS_MANAGER_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_EXTENSIONS_MANAGER, EphyExtensionsManagerPrivate))
 
 struct _EphyExtensionsManagerPrivate
@@ -80,13 +62,8 @@ struct _EphyExtensionsManagerPrivate
 	GList *extensions;
 	GList *dir_monitors;
 	GList *windows;
-	guint active_extensions_notifier_id;
 	guint sync_timeout_id;
 	GHashTable *reload_hash;
-
-#ifdef ENABLE_LEGACY_FORMAT
-	xsltStylesheetPtr xml2ini_xsl;
-#endif
 };
 
 typedef struct
@@ -98,10 +75,6 @@ typedef struct
 
 	EphyLoader *loader; /* NULL if never loaded */
 	GObject *extension; /* NULL if unloaded */
-
-#ifdef ENABLE_LEGACY_FORMAT
-	guint is_legacy_format : 1;
-#endif
 } ExtensionInfo;
 
 typedef struct
@@ -114,9 +87,6 @@ typedef enum
 {
 	FORMAT_UNKNOWN,
 	FORMAT_INI
-#ifdef ENABLE_LEGACY_FORMAT
-	, FORMAT_XML
-#endif
 } ExtensionFormat;
 
 enum
@@ -137,43 +107,69 @@ G_DEFINE_TYPE_WITH_CODE (EphyExtensionsManager, ephy_extensions_manager, G_TYPE_
 			 G_IMPLEMENT_INTERFACE (EPHY_TYPE_EXTENSION,
 						ephy_extensions_manager_iface_init))
 
+static void
+ephy_extensions_manager_toggle_load (EphyExtensionsManager *manager,
+				     const char *identifier,
+				     gboolean status)
+{
+	char **exts;
+	GVariantBuilder builder;
+	int i;
+	gboolean found = FALSE;
+
+	g_return_if_fail (EPHY_IS_EXTENSIONS_MANAGER (manager));
+	g_return_if_fail (identifier != NULL);
+
+	if (status)
+		LOG ("Adding '%s' to extensions", identifier);
+	else
+		LOG ("Removing '%s' from extensions", identifier);
+
+	exts = g_settings_get_strv (EPHY_SETTINGS_MAIN,
+				    EPHY_PREFS_ENABLED_EXTENSIONS);
+
+	g_variant_builder_init (&builder, G_VARIANT_TYPE_STRING_ARRAY);
+	for (i = 0; exts[i]; i++)
+	{
+		/* Ignore the extension if we are removing it. */
+		if (g_strcmp0 (exts[i], identifier) == 0)
+		{
+			found = TRUE;
+			if (!status)
+				continue;
+		}
+
+		g_variant_builder_add (&builder, "s", exts[i]);
+	}
+
+	if (status && !found)
+		g_variant_builder_add (&builder, "s", identifier);
+
+	g_settings_set (EPHY_SETTINGS_MAIN,
+			EPHY_PREFS_ENABLED_EXTENSIONS,
+			"as", &builder);
+}
+
 /**
  * ephy_extensions_manager_load:
  * @manager: an #EphyExtensionsManager
- * @name: identifier of the extension to load
+ * @identifier: identifier of the extension to load
  *
- * Loads the @name extension.
+ * Loads the extension corresponding to @identifier.
  **/
 void
 ephy_extensions_manager_load (EphyExtensionsManager *manager,
 			      const char *identifier)
 {
-	GSList *gconf_exts;
-
-	g_return_if_fail (EPHY_IS_EXTENSIONS_MANAGER (manager));
-	g_return_if_fail (identifier != NULL);
-
-	LOG ("Adding '%s' to extensions", identifier);
-
-	gconf_exts = eel_gconf_get_string_list (CONF_LOADED_EXTENSIONS);
-
-	if (!g_slist_find_custom (gconf_exts, identifier, (GCompareFunc) strcmp))
-	{
-		gconf_exts = g_slist_prepend (gconf_exts, g_strdup (identifier));
-
-		eel_gconf_set_string_list (CONF_LOADED_EXTENSIONS, gconf_exts);
-	}
-
-	g_slist_foreach (gconf_exts, (GFunc) g_free, NULL);
-	g_slist_free (gconf_exts);
+	ephy_extensions_manager_toggle_load (manager, identifier, TRUE);
 }
 
 /**
  * ephy_extensions_manager_unload:
  * @manager: an #EphyExtensionsManager
- * @name: filename of extension to unload, minus "lib" and "extension.so"
+ * @identifier: filename of extension to unload, minus "lib" and "extension.so"
  *
- * Unloads the extension specified by @name.
+ * Unloads the extension specified by @identifier.
  *
  * The extension with the same filename can afterwards be reloaded. However,
  * if any GTypes within the extension have changed parent types, Epiphany must
@@ -183,29 +179,7 @@ void
 ephy_extensions_manager_unload (EphyExtensionsManager *manager,
 				const char *identifier)
 {
-	GSList *gconf_exts;
-	GSList *l;
-
-	g_return_if_fail (EPHY_IS_EXTENSIONS_MANAGER (manager));
-	g_return_if_fail (identifier != NULL);
-
-	LOG ("Removing '%s' from extensions", identifier);
-
-	gconf_exts = eel_gconf_get_string_list (CONF_LOADED_EXTENSIONS);
-
-	l = g_slist_find_custom (gconf_exts, identifier, (GCompareFunc) strcmp);
-
-	if (l != NULL)
-	{
-		gconf_exts = g_slist_remove_link (gconf_exts, l);
-		g_free (l->data);
-		g_slist_free_1 (l);
-
-		eel_gconf_set_string_list (CONF_LOADED_EXTENSIONS, gconf_exts);
-	}
-
-	g_slist_foreach (gconf_exts, (GFunc) g_free, NULL);
-	g_slist_free (gconf_exts);
+	ephy_extensions_manager_toggle_load (manager, identifier, FALSE);
 }
 
 /**
@@ -234,7 +208,8 @@ ephy_extensions_manager_register (EphyExtensionsManager *manager,
  *
  * Returns the list of known extensions.
  *
- * Returns: a list of #EphyExtensionInfo
+ * Returns: (element-type EphyEmbed) (transfer container): a list of
+ *          #EphyExtensionInfo
  **/
 GList *
 ephy_extensions_manager_get_extensions (EphyExtensionsManager *manager)
@@ -359,77 +334,6 @@ ephy_extensions_manager_load_ini_file (EphyExtensionsManager *manager,
 	ephy_extensions_manager_parse_keyfile (manager, keyfile, identifier);
 }
 
-#ifdef ENABLE_LEGACY_FORMAT
-
-static void
-ephy_extensions_manager_load_xml_file (EphyExtensionsManager *manager,
-				       const char *identifier,
-				       const char *path)
-{
-	EphyExtensionsManagerPrivate *priv = manager->priv;
-	ExtensionInfo *info;
-	xmlDocPtr doc, res;
-	const xmlChar *xsl_file;
-	xmlChar *output = NULL;
-	int outlen = -1, ret = -1;
-	
-	START_PROFILER ("Transforming .xml -> " DOT_INI)
-
-	doc = xmlParseFile (path);
-	if (!doc) goto out;
-
-	if (priv->xml2ini_xsl == NULL)
-	{
-		xsl_file = (const xmlChar *) ephy_file ("ephy-xml2ini.xsl");
-		if (!xsl_file) return;
-
-		priv->xml2ini_xsl = xsltParseStylesheetFile (xsl_file);
-		if (priv->xml2ini_xsl == NULL)
-		{
-			g_warning ("Couldn't parse the XSL to transform .xml extension descriptions!\n");
-			goto out;
-		}
-	}
-
-	res = xsltApplyStylesheet (priv->xml2ini_xsl, doc, NULL);
-	if (!res) goto out;
-
-	ret = xsltSaveResultToString (&output, &outlen, res, priv->xml2ini_xsl);
-
-	if (ret >= 0 && output != NULL && outlen > -1)
-	{
-		GKeyFile *keyfile;
-		GError *err = NULL;
-
-		keyfile = g_key_file_new ();
-		if (!g_key_file_load_from_data (keyfile, (char *) output, outlen,
-					        G_KEY_FILE_NONE, &err))
-		{
-			g_warning ("Could load converted key file for '%s': '%s'",
-				   identifier, err->message);
-			g_error_free (err);
-			g_key_file_free (keyfile);
-			goto out;
-		}
-
-		info = ephy_extensions_manager_parse_keyfile (manager, keyfile, identifier);
-		if (info != NULL)
-		{
-			info->is_legacy_format = TRUE;
-		}
-	}
-
-	xmlFreeDoc (res);
-	xmlFreeDoc (doc);
-
-out:
-	xmlFree (output);
-
-	STOP_PROFILER ("Transforming .xml -> " DOT_INI)
-}
-
-#endif /* ENABLE_LEGACY_FORMAT */
-
 static char *
 path_to_identifier (const char *path)
 {
@@ -438,13 +342,6 @@ path_to_identifier (const char *path)
 	identifier = g_path_get_basename (path);
 	dot = strstr (identifier, DOT_INI);
 
-#ifdef ENABLE_LEGACY_FORMAT
-	if (!dot)
-	{
-		dot = strstr (identifier, ".xml");
-	}
-#endif
-
 	g_return_val_if_fail (dot != NULL, NULL);
 
 	*dot = '\0';
@@ -461,12 +358,6 @@ format_from_path (const char *path)
 	{
 		format = FORMAT_INI;
 	}
-#ifdef ENABLE_LEGACY_FORMAT
-	else if (g_str_has_suffix (path, ".xml"))
-	{
-		format = FORMAT_XML;
-	}
-#endif
 
 	return format;
 }
@@ -490,19 +381,8 @@ ephy_extensions_manager_load_file (EphyExtensionsManager *manager,
 				      (GCompareFunc) find_extension_info);
 	if (element != NULL)
 	{
-#ifdef ENABLE_LEGACY_FORMAT
-		ExtensionInfo *info = (ExtensionInfo *) element->data;
-
-		/* If this is the legacy format and we already have the info
-		 * read for this type from a non-legacy format file, don't
-		 * warn.
-		 */
-		if (format == FORMAT_XML && !info->is_legacy_format)
-#endif
-		{
-			g_warning ("Extension description for '%s' already read!",
-				   identifier);
-		}
+		g_warning ("Extension description for '%s' already read!",
+			   identifier);
 
 		g_free (identifier);
 		return;
@@ -513,13 +393,6 @@ ephy_extensions_manager_load_file (EphyExtensionsManager *manager,
 		ephy_extensions_manager_load_ini_file (manager, identifier,
 						       path);
 	}
-#ifdef ENABLE_LEGACY_FORMAT
-	else if (format == FORMAT_XML)
-	{
-		ephy_extensions_manager_load_xml_file (manager, identifier,
-						       path);
-	}
-#endif
 
 	g_free (identifier);
 }
@@ -580,20 +453,23 @@ get_loader_for_type (EphyExtensionsManager *manager,
 	}
 	if (strcmp (type, "python") == 0)
 	{
-#ifdef ENABLE_PYTHON
+		return NULL;
+	}
+
+#ifdef ENABLE_SEED
+	if (strcmp (type, "seed") == 0)
+	{
 		info = g_new (LoaderInfo, 1);
 		info->type = g_strdup (type);
-		info->loader = g_object_new (EPHY_TYPE_PYTHON_LOADER, NULL);
+		info->loader = g_object_new (EPHY_TYPE_SEED_LOADER, NULL);
 
 		manager->priv->factories =
 				g_list_append (manager->priv->factories, info);
 
 		return g_object_ref (info->loader);
-#else
 		return NULL;
-#endif
 	}
-
+#endif
 	shlib_loader = get_loader_for_type (manager, "shlib");
 	g_return_val_if_fail (shlib_loader != NULL, NULL);
 
@@ -754,50 +630,65 @@ unload_extension (EphyExtensionsManager *manager,
 static void
 sync_loaded_extensions (EphyExtensionsManager *manager)
 {
-	GConfClient *client;
-	GConfValue *value;
-	GSList *active_extensions = NULL;
+	char **extensions;
+	GVariantBuilder builder;
+	int i;
+	gboolean has_ui = FALSE;
 	GList *l;
-	gboolean active;
 	ExtensionInfo *info;
 
 	LOG ("Synching changed list of active extensions");
 
-	client = gconf_client_get_default ();
-	g_return_if_fail (client != NULL);
+	extensions = g_settings_get_strv (EPHY_SETTINGS_MAIN,
+					  EPHY_PREFS_ENABLED_EXTENSIONS);
 
-	value = gconf_client_get (client, CONF_LOADED_EXTENSIONS, NULL);
+	g_variant_builder_init (&builder, G_VARIANT_TYPE_STRING_ARRAY);
 
-	/* make sure the extensions-manager-ui is loaded */
-	if (value == NULL ||
-	    value->type != GCONF_VALUE_LIST ||
-	    gconf_value_get_list_type (value) != GCONF_VALUE_STRING)
+	/* Make sure the extensions-manager-ui is always loaded. */
+	for (i = 0; extensions[i]; i++)
 	{
-		active_extensions = g_slist_prepend (active_extensions,
-						     g_strdup ("extensions-manager-ui"));
-		eel_gconf_set_string_list (CONF_LOADED_EXTENSIONS, active_extensions);
+		if (g_strcmp0 (extensions[i], "extensions-manager-ui") == 0)
+			has_ui = TRUE;
+
+		g_variant_builder_add (&builder, "s", extensions[i]);
 	}
-	else
+
+	if (!has_ui)
 	{
-		active_extensions = eel_gconf_get_string_list (CONF_LOADED_EXTENSIONS);
+		g_variant_builder_add (&builder, "s", "extensions-manager-ui");
+		g_settings_set (EPHY_SETTINGS_MAIN,
+				EPHY_PREFS_ENABLED_EXTENSIONS,
+				"as", &builder);
+
+		g_strfreev (extensions);
+		extensions = g_settings_get_strv
+					(EPHY_SETTINGS_MAIN,
+					 EPHY_PREFS_ENABLED_EXTENSIONS);
 	}
 
+	g_variant_builder_end (&builder);
+
 	for (l = manager->priv->data; l != NULL; l = l->next)
 	{
 		gboolean changed;
+		gboolean active = FALSE;
+		int j;
 		
 		info = (ExtensionInfo *) l->data;
 
-		active = (g_slist_find_custom (active_extensions,
-					       info->info.identifier,
-					       (GCompareFunc) strcmp) != NULL);
+		for (j = 0; extensions[j]; j++)
+		{
+			if (!active && g_strcmp0 (extensions[j],
+						  info->info.identifier) == 0)
+				active = TRUE;
+		}
 
 		LOG ("Extension '%s' is %sactive and %sloaded",
 		     info->info.identifier,
 		     active ? "" : "not ",
 		     info->info.active ? "" : "not ");
 
-		changed = ( info->info.enabled != active );
+		changed = (info->info.enabled != active);
 
 		info->info.enabled = active;
 
@@ -824,14 +715,7 @@ sync_loaded_extensions (EphyExtensionsManager *manager)
 		}
 	}
 
-	g_slist_foreach (active_extensions, (GFunc) g_free, NULL);
-	g_slist_free (active_extensions);
-
-	if (value != NULL)
-	{
-		gconf_value_free (value);
-	}
-	g_object_unref (client);
+	g_strfreev (extensions);
 }
 
 static void
@@ -995,9 +879,10 @@ static void
 ephy_extensions_manager_load_dir (EphyExtensionsManager *manager,
 				  const char *path)
 {
-	DIR *d;
-	struct dirent *e;
 	char *file_path;
+	GError *error = NULL;
+	GDir *dir;
+	const char *dir_elem;
 	GFile *directory;
 	GFileMonitor *monitor;
 
@@ -1005,21 +890,28 @@ ephy_extensions_manager_load_dir (EphyExtensionsManager *manager,
 
 	START_PROFILER ("Scanning directory")
 
-	d = opendir (path);
-	if (d == NULL)
+	dir = g_dir_open (path, 0, &error);
+	if (error)
 	{
+		LOG ("Failed to open extension directory %s: %s",
+		     path, error->message);
+		g_error_free (error);
 		return;
 	}
-	while ((e = readdir (d)) != NULL)
+
+	dir_elem = g_dir_read_name (dir);
+	while (dir_elem)
 	{
-		if (format_from_path (e->d_name) != FORMAT_UNKNOWN)
+		if (format_from_path (dir_elem) != FORMAT_UNKNOWN)
 		{
-			file_path = g_build_filename (path, e->d_name, NULL);
+			file_path = g_build_filename (path, dir_elem, NULL);
 			ephy_extensions_manager_load_file (manager, file_path);
 			g_free (file_path);
 		}
+
+		dir_elem = g_dir_read_name (dir);
 	}
-	closedir (d);
+	g_dir_close (dir);
 
 	directory = g_file_new_for_path (path);
 	monitor = g_file_monitor_directory (directory, 0, NULL, NULL);
@@ -1037,10 +929,9 @@ ephy_extensions_manager_load_dir (EphyExtensionsManager *manager,
 }
 
 static void
-active_extensions_notifier (GConfClient *client,
-			    guint cnxn_id,
-			    GConfEntry *entry,
-			    EphyExtensionsManager *manager)
+active_extensions_cb (GSettings *settings,
+		      char *key,
+		      EphyExtensionsManager *manager)
 {
 	sync_loaded_extensions (manager);
 }
@@ -1081,12 +972,12 @@ ephy_extensions_manager_startup (EphyExtensionsManager *manager)
 
 	ephy_extensions_manager_load_dir (manager, EXTENSIONS_DIR);
 
-	active_extensions_notifier (NULL, 0, NULL, manager);
-	manager->priv->active_extensions_notifier_id =
-		eel_gconf_notification_add
-			(CONF_LOADED_EXTENSIONS,
-			 (GConfClientNotifyFunc) active_extensions_notifier,
-			 manager);
+	sync_loaded_extensions (manager);
+
+	g_signal_connect (EPHY_SETTINGS_MAIN,
+			  "changed::" EPHY_PREFS_ENABLED_EXTENSIONS,
+			  G_CALLBACK (active_extensions_cb),
+			  manager);
 }
 
 static void
@@ -1095,20 +986,6 @@ ephy_extensions_manager_dispose (GObject *object)
 	EphyExtensionsManager *manager = EPHY_EXTENSIONS_MANAGER (object);
 	EphyExtensionsManagerPrivate *priv = manager->priv;
 
-#ifdef ENABLE_LEGACY_FORMAT
-	if (priv->xml2ini_xsl != NULL)
-	{
-		xsltFreeStylesheet (priv->xml2ini_xsl);
-		priv->xml2ini_xsl = NULL;
-	}
-#endif
-
-	if (priv->active_extensions_notifier_id != 0)
-	{
-		eel_gconf_notification_remove (priv->active_extensions_notifier_id);
-		priv->active_extensions_notifier_id = 0;
-	}
-
 	if (priv->reload_hash != NULL)
 	{
 		g_hash_table_destroy (priv->reload_hash);
